C***********************************************************************
C***********************************************************************
C                                                                      *
C                  -    "MADBEND"   -                                  *
C                                                                      *
C          A  PROGRAM FOR THE MACROSCOPIC ANALYSIS OF                  *
C          DNA BENDING ANGLES, YIELDING THE VALUES OF                  * 
C          THE GLOBAL TILT AND GLOBAL ROLL BEND ANGLES                 *
C                                                                      *
C                copyright   (c)   1999  by                            *
C                                                                      *
C            Tamar Schlick   &    Daniel Strahs                        *
c                                                                      *
C  For details, see                                                    *
C                                                                      *
C    (1) Strahs, D. and Schlick, T.                                    *
C        A-tract Bending: Insights into Experimental                   *
C	 Structures by Computational Models                            *
C	 Journal of Molecular Biology  (2000)                          *
C                                                                      *
C  The manuscript is also available from the authors. Please send      *
C  requests by email to                                                *
C                                                                      *
C      dan.strahs@nyu.edu    or   schlick@nyu.edu                      *
C                                                                      *
C  Alternatively, both this software and the manuscript are            *
C  downloadable from the Schlick Group Web Site:                       *
C                                                                      *
C           http:\\monod.biomath.nyu.edu                               *
C                                                                      *
C***********************************************************************
C                                                                      *
C PLEASE NOTE:                                                         *
C                                                                      *
C (i)   Double Precision is used throughout.                           *
C                                                                      *
C (ii)  Documentation for the program is given separately;             *
C       only brief comments are provided here. The program             *
C       documentation is supplied in the distribution.                 *
C       Alternatively, the documentation may be downloaded from        *
C              http:\\monod.biomath.nyu.edu                            *
C                                                                      *
C (iii) Madbend requires nucleic acid structural parameters            *
C	calculated by a separate program                               *
C                                                                      *
C***********************************************************************
C                                                                      *
C MADBEND COMPONENTS:                                                  *
C                                                                      *
C (A) MADBEND SUBROUTINES (IN APPROXIMATE CALLING ORDER):              *
C     ----------------------------                                     *
C                                                                      *
C  *   PARSFLG    		- parse command line flags             *
C                                 and program inputs;                  *
C                                 also, initialize some variables      *
C  *   AFTPARS          	- ask for undefined input arguments    *
C                                 not specified on the command line    *
C  *   USAGE      		- usage message; brief program         *
C                                 description                          *
C  *   READINP   		- central subroutine; determines       *
C                                 format type of input nucleic acid    *
C                                 parameter file and directs to        *
C                                 appropriate I/O subroutine           * 
C  *   READ3CL        		- I/O subroutine                       *
C                                 for three-column format              *
C  *   READCUR    		- I/O subroutine                       *
C                                 for Curves file format               *
C  *   EXCLUDE      		- decode parameters specifying         *
C                                 bp steps to be excluded              *
C  *   ACTWIST			- this is the main subroutine.         *
C                                 1) determines ref. plane position;   *
C                                 2) calls EXCLUDE subroutine;         *
C                                 3) partition tilt and roll terms,    *
C                                    ignoring excluded bp steps; and   *
C                                 4) accumulates twist values.         *
C  *   CALGTGR  		- calculate global tilt/global roll    *
C  *   OUTGTGR    		- results : global tilt/global roll    *
C  *   OPENFIL			- generalized file opening subroutine  *
C                                                                      *
C (B) MADBEND FILES:                                                   *
C     --------------                                                   *
C                                                                      *
C     MADBEND.F                 - central program                      *
C                                                                      *
C     PARAM.H                   - include file for various commonly    * 
C                                 used variable                        *
C                                                                      *
C     MAKEFILE                  - the makefile                         *
C                                                                      *
C***********************************************************************
C                                                                      *
C Aug. 1 2000 - Bugfix in ACTWIST for reference planes lying exactly   *
C               on base pairs; bug identified by Sylvie Derreumaux     *
C                                                                      *
C Apr. 24 2002 - Bugfixes in ACTWIST for intrinsic twist of reference  *
C               plane; bug identified by Andrej Barbic/Donald Crothers *
C                                                                      *
C***********************************************************************
C***********************************************************************
      PROGRAM MADBEND


      IMPLICIT NONE
      INCLUDE 'param.h'

C --------------------------------------------------------
C INPUT ARGUMENTS TO PROGRAM; EXAMINE COMMAND LINE
C FOR INPUT ARGUMENTS - THEN SEARCH FOR ANY UNDEFINED
C ARGUMENTS BY ASKING QUESTIONS OF THE USER 
C --------------------------------------------------------
      CALL PARSFLG    

      CALL AFTPARS

C --------------------------------------------------------
C LOAD DATA; IF QCURVES IS TRUE, THEN THE PROGRAM
C HAS DETECTED A ``CURVES'' TYPE INPUT FILE.
C ``CURVES'' INPUT FILES AND THE REQUIRED FORMAT ARE
C DESCRIBED IN THE ACCOMPANYING DOCUMENTATION.
C IF QCURVES IS FALSE, THEN THE FILE IS ASSUMED TO 
C BE AN ALTERNATE, THREE-COLUMN FORMAT.
C --------------------------------------------------------
      IF (QCURVES.EQ.1) THEN
         CALL READCUR    
      ELSE
         CALL READ3CL        
      ENDIF

C --------------------------------------------------------
C AFTER THE DATA IS LOADED, WE DO MOST OF THE WORK.
C THIS SUBROUTINE DETERMINES THE POSITION OF THE
C REFERENCE BASE PAIR PLANE, DETERMINES WHICH TERMS
C (IF ANY) ARE TO BE EXCLUDED, PARTITIONS TILT AND
C ROLL TERMS RELATIVE TO THE REFERENCE PLANE, AND
C ACCUMULATES THE TWIST TERMS.
C --------------------------------------------------------
      CALL ACTWIST

C --------------------------------------------------------
C CALCULATE GLOBAL TILT AND GLOBAL ROLL 
C --------------------------------------------------------
      CALL CALGTGR  

C --------------------------------------------------------
C OUTPUT RESULTS
C --------------------------------------------------------
      CALL OUTGTGR    


999   CONTINUE
      STOP
      END

C****************************************************************
C****************************************************************

      SUBROUTINE CALGTGR  

C --------------------------------------------------------
C THIS SUBROUTINE DOES THE ACTUAL CALCULATION OF
C GLOBAL TILT AND GLOBAL ROLL. THE CENTRAL EQUATION
C OF THIS SUBROUTINE IS DETAILED AS EQUATION (2) IN THE
C ACCOMPANYING DOCUMENTATION.
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER I
      DOUBLE PRECISION CONV
      INCLUDE 'param.h'

C --------------------------------------------------------
C CONV: CONVERSION FACTOR FROM DEGREES TO RADIANS
C --------------------------------------------------------
      CONV = PI / ONE8TY
C --------------------------------------------------------
C APPLY A TWO-DIMENSIONAL ROTATION MATRIX TO 
C CALCULATE GLOBAL TILT (GT) AND GLOBAL ROLL (GR).
C THE ROTATION ANGLE OF THE MATRIX IS THE ACCUMULATED TWIST
C FROM THE SUBROUTINE ACTWIST.
C --------------------------------------------------------
      DO I = 1, NUMTERM
        IF (QLOCAL.EQ.1) THEN
         GTL(I) = (LNTILT(I) * COS(LACCTWS(I)*CONV)) + 
     *			(LNROLL(I) * SIN(LACCTWS(I)*CONV))
         GRL(I) = (LNTILT(I) * SIN(LACCTWS(I)*CONV) * NEGONE) +
     *                  (LNROLL(I) * COS(LACCTWS(I)*CONV))
        ENDIF
        IF (QGLOBAL.EQ.1) THEN
         GTG(I) = (GNTILT(I) * COS(GACCTWS(I)*CONV)) +
     *                  (GNROLL(I) * SIN(GACCTWS(I)*CONV))
         GRG(I) = (GNTILT(I) * SIN(GACCTWS(I)*CONV) * NEGONE) +
     *                  (GNROLL(I) * COS(GACCTWS(I)*CONV))
        ENDIF
      END DO

CDEBUG      DO I=1,NUMTERM
CDEBUG        WRITE(6,*) GTL(I), GRL(I)
CDEBUG      END DO

      RETURN
      END

C***************************************************************
C***************************************************************

      SUBROUTINE ACTWIST

C --------------------------------------------------------
C THIS IS THE CENTRAL SUBROUTINE OF THE MADBEND PROGRAM.
C 1) ESTABLISH THE POSITION OF THE REFERENCE PLANE
C 2) EXAMINE THE EXCLUDED TERMS TO DETERMINE
C    WHICH BP STEPS ARE TO BE EXCLUDED 
C 3) PARTITION TILT AND ROLL TERMS TO NEW ARRAYS,
C    ALLOWING FOR THE REFERENCE PLANE POSITION
C    AND EXCLUDING INDICATED BP STEPS
C 4) ACCUMULATE THE CUMULATIVE TWIST VALUES, 
C    REVERSING THE SIGN OF CUMULATIVE TWIST BELOW THE REFERENCE PLANE. 
C --------------------------------------------------------


      IMPLICIT NONE
      INTEGER IPOSN, I,J, IOFFSET 
      DOUBLE PRECISION FRACTIN, PORTION, TWPORTION
      INCLUDE 'param.h'

C --------------------------------------------------------
C DECODE REFERENCE PLANE POSITION
C THE REFERENCE PLANE POSITION IS INPUT INTO THE PROGRAM
C AS EITHER AN INTEGER OR AS A REAL NUMBER OF THE FORM
C  IPOSN.FRACTIN
C NOTE THAT A REF OF 5.0 COINCIDES WITH THE 4TH BP STEP (BP 5)
C  WHILE A REF OF 4.2 IS 20%/80% BETWEEN BP 4 AND BP 5 
C --------------------------------------------------------
      IPOSN = REF
      FRACTIN = REF - (IPOSN*ONE)
      IPOSN = IPOSN - 1

CDEBUG      WRITE(6,*) IPOSN, FRACTIN, REF
     
C --------------------------------------------------------
C ESTABLISH TOLERANCE LIMIT: IF THE FRACTIONAL POSITION
C OF THE REFERENCE PLANE IS LESS THAN EPSILON FROM SOME
C BASE PAIR, THE REFERENCE PLANE POSITION IS ASSIGNED 
C TO THAT BASE PAIR.
C THE VALUE OF EPSILON IS SET IN PARAM.H. 
C --------------------------------------------------------
      IF (FRACTIN.LE.EPSILON) FRACTIN = ZERO
      IF ((ONE-FRACTIN).LE.EPSILON) THEN
          FRACTIN = ZERO
          IPOSN = IPOSN + 1
      ENDIF

CDEBUG      WRITE(6,*) IPOSN, FRACTIN, REF

C --------------------------------------------------------
C THE NUMBER OF TERMS (NUMTERM) IS EQUAL TO THE NUMBER OF BP STEPS;
C HOWEVER, IF THE REFERENCE PLANE PARAMETER FRACTIN IS NONZERO,
C  THEN THE REFERENCE PLANE DIVIDES A BP STEP AND INCREASES
C  THE NUMBER OF BP STEPS
C --------------------------------------------------------
      NUMTERM = NUMSTEP
      IF (FRACTIN.NE.ZERO) NUMTERM = NUMSTEP + 1

C --------------------------------------------------------
C IF BP STEPS HAVE NOT BEEN EXCLUDED FROM THE GT/GR SUMS,
C  CALL EXCLUDE AND PROCESS THE RANGE OF EXCLUSION TERMS. 
C THIS SETS EXCLTLT/EXCLROL FOR THOSE TERMS EQUAL TO ZERO.  
C OTHERWISE, EXCLTLT/EXCLROL IS EQUAL TO ONE.  
C -------------------------------------------------------- 

      IF (QEXCL.EQ.0) CALL EXCLUDE      

C -------------------------------------------------------- 
C PARTITION TILT, ROLL AND TWIST TERMS      
C  IOFFSET: OFFSET OF TERM INDICES IF REFERENCE PLANE IS BETWEEN BPS. 
C    IOFFSET IS DIFFERENCE BETWEEN THE BP STEP INDEX 
C      AND THE GT/GR TERM INDEX .  
C  IF BP STEP IS ABOVE THE REFERENCE PLANE, IOFFSET = 0
C  IF BP STEP IS BELOW THE REFERENCE PLANE, IOFFSET = -1
C
C  EXCLTLT OR EXCLROL: 1 OR 0, EXCLUDING EITHER TILT OR ROLL 
C   EXCLTLT AND EXCLROL ARE MULTIPLIED 
C   AGAINST TILT AND ROLL, RESPECTIVELY.
C --------------------------------------------------------

      IOFFSET = 0
      DO I = 1, NUMTERM
        IF (FRACTIN.NE.ZERO.AND.I.GT.IPOSN+1) IOFFSET = -1
        IF (FRACTIN.EQ.ZERO.OR.I.LT.IPOSN.OR.I.GT.IPOSN+1) THEN
          IF (QGLOBAL.EQ.1) THEN
            GNTILT(I) = GTILT(I+IOFFSET)*EXCLTLT(I+IOFFSET)
            GNROLL(I) = GROLL(I+IOFFSET)*EXCLROL(I+IOFFSET)
            GNTWIST(I) = GTWIST(I+IOFFSET)
          ENDIF
          IF (QLOCAL.EQ.1) THEN
            LNTILT(I) = LTILT(I+IOFFSET)*EXCLTLT(I+IOFFSET)
            LNROLL(I) = LROLL(I+IOFFSET)*EXCLROL(I+IOFFSET)
            LNTWIST(I) = LTWIST(I+IOFFSET)
          ENDIF
        ENDIF
C --------------------------------------------------------
C  PORTION: FRACTIN OR (1 - FRACTIN)
C   PORTION IS USED TO DIVIDE THE TILT AND ROLL VALUES
C   OF THE FRACTIONAL BP STEP TO THE TWO HALVES OF THE FRACTIONAL STEP.
C  TWPORTION: USED TO REMOVE THE TWIST OF THE REFERENCE BP STEP.
C --------------------------------------------------------

        IF ((I.EQ.IPOSN.OR.I.EQ.IPOSN+1).OR.
     *      (FRACTIN.NE.ZERO.AND.I.EQ.IPOSN+2)) THEN

          PORTION = FRACTIN
          TWPORTION = FRACTIN
          IF ((I.EQ.IPOSN+2.AND.FRACTIN.NE.ZERO).OR.
     *        (I.EQ.IPOSN+1.AND.FRACTIN.EQ.ZERO)) THEN
             PORTION = (ONE - FRACTIN) 
             TWPORTION = (ONE - FRACTIN)
          ENDIF
          IF (I.EQ.IPOSN) PORTION = ONE
          IF (I.EQ.IPOSN+1.AND.FRACTIN.NE.ZERO) TWPORTION = ZERO

          IF (QGLOBAL.EQ.1) THEN
            GNTILT(I) = GTILT(I+IOFFSET)*PORTION*EXCLTLT(I+IOFFSET)
            GNROLL(I) = GROLL(I+IOFFSET)*PORTION*EXCLROL(I+IOFFSET)
            GNTWIST(I) = GTWIST(I+IOFFSET)*TWPORTION
          ENDIF
          IF (QLOCAL.EQ.1) THEN
            LNTILT(I) = LTILT(I+IOFFSET)*PORTION*EXCLTLT(I+IOFFSET)
            LNROLL(I) = LROLL(I+IOFFSET)*PORTION*EXCLROL(I+IOFFSET)
            LNTWIST(I) = LTWIST(I+IOFFSET)*TWPORTION
          ENDIF
        ENDIF
      END DO


CDEBUG      DO I = 1, NUMSTEP
CDEBUG       WRITE(6,*) GTILT(I), GROLL(I)
CDEBUG      END DO
CDEBUG
CDEBUG      DO I = 1, NUMTERM
CDEBUG       WRITE(6,'(I2,3(1x,F6.2))') I,LNTILT(I),LNROLL(I),LNTWIST(I)
CDEBUG      END DO

C --------------------------------------------------------
C REVERSE SIGN OF TWIST TERMS BELOW REFERENCE PLANE
C THIS IS THE CONVENTION WE USE; TWIST BELOW THE REFERENCE PLANE
C IS ALWAYS NEGATIVE IF RIGHT-HANDED.
C --------------------------------------------------------
      DO I = 1, NUMTERM
        IF (I.GE.IPOSN-IOFFSET) THEN
          IF (QGLOBAL.EQ.1) GNTWIST(I) = GNTWIST(I) * NEGONE
          IF (QLOCAL.EQ.1) LNTWIST(I) = LNTWIST(I) * NEGONE
        ENDIF
      END DO

CDEBUG      DO I = 1, NUMTERM
CDEBUG       WRITE(6,*) LNTILT(I), LNROLL(I), LNTWIST(I)
CDEBUG      END DO

C --------------------------------------------------------
C ACCUMULATE TWIST TERMS. 
C THE TWIST TERMS ABOVE AND BELOW THE REFERENCE PLANE
C  NEED TO BE ACCUMULATED SEPARATELY DUE TO THE SIGN REVERSAL
C --------------------------------------------------------
C ABOVE THE REFERENCE PLANE
C --------------------------------------------------------
      DO I = 1, IPOSN-IOFFSET-1
          IF (QGLOBAL.EQ.1) GACCTWS(I) = ZERO
          IF (QLOCAL.EQ.1) LACCTWS(I) = ZERO
        DO J = IPOSN-IOFFSET-1, I, -1
          IF (QGLOBAL.EQ.1) GACCTWS(I) = GACCTWS(I) + GNTWIST(J)
          IF (QLOCAL.EQ.1) LACCTWS(I) = LACCTWS(I) + LNTWIST(J)
        END DO
      END DO
C --------------------------------------------------------
C BELOW THE REFERENCE PLANE
C --------------------------------------------------------
      DO I = IPOSN-IOFFSET, NUMTERM
          IF (QGLOBAL.EQ.1) GACCTWS(I) = ZERO
          IF (QLOCAL.EQ.1) LACCTWS(I) = ZERO
        DO J = IPOSN-IOFFSET, I
          IF (QGLOBAL.EQ.1) GACCTWS(I) = GACCTWS(I) + GNTWIST(J)
          IF (QLOCAL.EQ.1) LACCTWS(I) = LACCTWS(I) + LNTWIST(J)
        END DO
      END DO

CDEBUG      DO I = 1, NUMTERM
CDEBUG       WRITE(6,'(I5,3X,F11.6,5X,F11.6,F11.6,F11.6)')
CDEBUG     *          I,LNTILT(I),LNROLL(I),LNTWIST(I),LACCTWS(I)
CDEBUG      END DO

      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE EXCLUDE      

C --------------------------------------------------------
C THIS SUBROUTINE DETERMINES IF BP STEPS ARE TO BE EXCLUDED
C  FROM THE GLOBAL TILT/GLOBAL ROLL SUMS.
C EXCLUDED BP STEPS CAN INCLUDE ANY COMBINATION OF TILT
C  AND/OR ROLL TERMS FROM ANY OF THE BP STEPS.
C POTENTIAL USES INCLUDE NEGLECTING END BP STEPS OR
C  DECOMPOSING THE RESULTING TOTAL BEND INTO INDIVIDUAL CONTRIBUTIONS.
C
C NOTE THAT THIS CODE IS FUNCTIONALLY DUPLICATED 
C  IN THE SUBROUTINE PARSFLG.
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER I,J,ITERM,IEXCL
      CHARACTER*1 YESNO
      CHARACTER*120 BUFFER
      INCLUDE 'param.h'

      NUMEXCL = 0
      WRITE(6,*)
      WRITE(6,'(A26,1X,I5)') 
     *		'NUMBER OF BP STEPS FOUND: ', NUMSTEP 
      WRITE(6,'(A39,$)') 
     *		'ARE BP STEPS TO BE EXCLUDED (CR = NO): '

      READ(5,'(A1)') YESNO
      IF (YESNO.EQ.' '.OR.YESNO.EQ.'n'.OR.YESNO.EQ.'N') GOTO 100

C --------------------------------------------------------
C  IF YESNO NE N, CONTINUE PARSING LIST OF EXCLUDED BPS
C --------------------------------------------------------
C --------------------------------------------------------
C CLEAN THE BUFFER FOR PARSING EXCLUSION LIST
C --------------------------------------------------------
      DO I = 1, LEN(BUFFER)
       BUFFER(I:I) = ' '
      END DO

C --------------------------------------------------------
C NUMEXCL: NUMBER OF EXCLUDED BP STEPS
C --------------------------------------------------------
      WRITE(6,'(A37,$)') 
     *          'HOW MANY BP STEPS WILL BE EXCLUDED ? '
      READ(5,'(I)') NUMEXCL

C --------------------------------------------------------
C DESCRIBE THE LIST FORMAT
C    EACH BP STEP CAN BE PRECEDED BY THE LETTER t OR r (FOR tilt/roll)
C             SEPARATED BY COMMA OR WHITESPACE.
C    FOR EXAMPLE: 
C      1,11 (OR 1 11) MEANS EXCLUDE BP STEPS 1 AND 11 (BOTH TILT AND ROLL)
C      t1,r11 = EXCLUDE ONLY TILT OF BP STEP 1, ROLL OF 11
C      1,r11 = EXCLUDE TILT AND ROLL 1, AND ROLL OF BP STEP 11
C --------------------------------------------------------
      IF (QVERBOS.EQ.1) THEN
       WRITE(6,*)
       WRITE(6,*) 
     *          " FORMAT OF LIST: Each bp step can be preceded ",
     *		"by the letter t or r (for tilt/roll), "
       WRITE(6,*) 
     *          "         separated by comma or whitespace."
       WRITE(6,*) 
     *          "     for example: 1,11 (or 1 11) means ",
     *		"exclude bp steps 1 and 11 (both tilt and roll)"
       WRITE(6,*) 
     * "          t1,r11 = exclude only tilt of bp step 1, roll or 11"
       WRITE(6,*) 
     *          "          1,r11 = ",
     *		"exclude tilt and roll 1, and roll of bp step 11"
      ENDIF
      WRITE(6,'(A52)') 
     *		'Enter numbers of excluded bp steps, CR when finished'

C --------------------------------------------------------
C READ BUFFER: THE LIST OF EXCLUDED TERMS
C --------------------------------------------------------
      READ(5,'(A120)') BUFFER

C --------------------------------------------------------
C PARSE BUFFER, THE LIST OF EXCLUDED TERMS 
C  ITERM: THE COUNT OF THE NUMBER OF EXCLUSION TERMS PARSED
C  I: INDEX TO BUFFER
C  J : INDEX TO FIRST POSITION OF CURRENT PARSED SEGMENT
C --------------------------------------------------------
      ITERM=1
      J = 1
      DO I = 1, LEN(BUFFER)
C --------------------------------------------------------
C IF COMMA OR SPACE FOUND, INDICATES END OF TERM; 
C	THIS MEANS WE SHOULD PARSE THE TERM JUST FOUND.
C NON-COMMA OR SPACE CHARACTERS ARE LUMPED IN WITH NUMERIC TERMS 
C       AND THUS WILL BE IGNORED.
C --------------------------------------------------------
         IF (BUFFER(I:I).EQ.','.OR.
     *        (BUFFER(I:I).EQ.' '.AND.ITERM.LT.NUMEXCL)) THEN
C --------------------------------------------------------
C IEXCL: NUMBER JUST PARSED OUT OF BUFFER; THIS IS THE BP STEP TO EXCLUDE
C --------------------------------------------------------
           IF (BUFFER(J:J).NE.'r'.AND.BUFFER(J:J).NE.'t') THEN
             READ(BUFFER(J:I-1),*) IEXCL
           ELSE
             READ(BUFFER(J+1:I-1),*) IEXCL
           ENDIF
           EXCLTLT(IEXCL) = ZERO
           EXCLROL(IEXCL) = ZERO
           IF (BUFFER(J:J).EQ.'r') EXCLTLT(IEXCL) = ONE
           IF (BUFFER(J:J).EQ.'t') EXCLROL(IEXCL) = ONE
           ITERM=ITERM+1
           J = I+1
         ENDIF
      END DO
C --------------------------------------------------------
C FINAL SECTION:
C THIS CODE HAS LEFT THE LAST LIST TERM UNREAD; READ IT NOW
C --------------------------------------------------------
      IF (BUFFER(J:J).NE.'r'.AND.BUFFER(J:J).NE.'t') THEN
        READ(BUFFER(J:I-1),*) IEXCL
      ELSE
        READ(BUFFER(J+1:I-1),*) IEXCL
      ENDIF
      EXCLTLT(IEXCL) = ZERO
      EXCLROL(IEXCL) = ZERO
      IF (BUFFER(J:J).EQ.'r') EXCLTLT(IEXCL) = ONE
      IF (BUFFER(J:J).EQ.'t') EXCLROL(IEXCL) = ONE
C --------------------------------------------------------
C WARNING IF FEWER EXCLUSION LIST TERMS FOUND THAN EXPECTED
C --------------------------------------------------------
      IF (ITERM.NE.NUMEXCL)
     *    WRITE(6,*) "ERROR: TOO FEW EXCLUSION LIST TERMS FOUND"

100   CONTINUE
      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE READINP   

C --------------------------------------------------------
C  THIS SUBROUTINE READS THE SPECIFIED INPUT FILE WHICH 
C   IS EXPECTED TO CONTAIN NUCLEIC ACID STRUCTURAL PARAMETERS.
C  IF THE SUBROUTINE FINDS THE WORD "CURVES" ON THE THIRD
C   LINE OF THE FILE, IT ASSUMES ``CURVES'' INPUT AND SETS
C   QCURVES TO BE TRUE.
C  OTHERWISE, IT ASSUMES A THREE-COLUMN FORMAT AND SETS 
C   QCURVES TO BE FALSE.
C
C  BOTH INPUT FORMATS ARE DESCRIBED IN THE DOCUMENTATION.
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER ICOUNT, IFLAG, I
      CHARACTER*90 BUFFER
      INCLUDE 'param.h'

      QCURVES = 0
C --------------------------------------------------------
C OPEN THE FILE
C --------------------------------------------------------
      CALL OPENFIL(10,INFILE,'FORMATTED','READ ')

C --------------------------------------------------------
C CHECK THE THIRD LINE OF FILE
C --------------------------------------------------------
      READ(10,'(A90)') BUFFER
      READ(10,'(A90)') BUFFER
      READ(10,'(A90)') BUFFER

C --------------------------------------------------------
C CHECK FOR CURVES KEYWORD; IF FOUND, QCURVES = 1 
C --------------------------------------------------------
      IF (BUFFER(1:19).EQ.'     ******  CURVES') QCURVES = 1
      IF (QVERBOS.EQ.1) THEN
       IF (QCURVES.EQ.1) 
     *	WRITE(6,'(A19,A120)') 'CURVES INPUT FILE: ',INFILE
       IF (QCURVES.EQ.0) 
     *	WRITE(6,'(A21,A120)') 'STANDARD INPUT FILE; ',INFILE
      ENDIF
C --------------------------------------------------------
C CLOSE THE FILE; IT WILL BE REOPENED LATER BY THE CORRECT SUBROUTINE.
C --------------------------------------------------------
      CLOSE(10)

C      IF (QCURVES.EQ.1) THEN
C         CALL READCUR    
C      ELSE
C         CALL READ3CL        
C      ENDIF

      CLOSE(10)
      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE READ3CL        

C --------------------------------------------------------
C THIS SUBROUTINE READS A 3 COLUMN FORMAT.
C THE FIRST LINE IS N, THE NUMBER OF BP STEPS
C THE NEXT N LINES HAVE 3 VALUES PER LINE.
C IN ORDER, THE 3 VALUES ARE TILT, ROLL, AND TWIST.
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER I
      CHARACTER*90 BUFFER
      INCLUDE 'param.h'

C --------------------------------------------------------
C OPEN THE FILE
C --------------------------------------------------------
      CALL OPENFIL(10,INFILE,'FORMATTED','READ ')

      NUMSTEP = 0
      I = 0
C --------------------------------------------------------
C FIRST LINE IS NUMBER OF BP STEPS
C --------------------------------------------------------
      READ(10,'(I)',END=990) I
      NUMSTEP = I

C --------------------------------------------------------
C READ BP STEP PARAMETERS
C --------------------------------------------------------
      DO I = 1, NUMSTEP
        READ(10,'(A90)',END=990) BUFFER
        READ(BUFFER,*) LTILT(I),LROLL(I),LTWIST(I)
      END DO
      GOTO 991

C --------------------------------------------------------
C END-OF-FILE ERROR MESSAGE
C --------------------------------------------------------
990   WRITE(6,*)
     *  "END OF FILE ERROR: ",I-1," PARAMETERS FOUND, ",
     *  NUMSTEP, " EXPECTED"
991   CONTINUE

      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE READCUR    

C --------------------------------------------------------
C THIS SUBROUTINE READS CURVES STYLE FORMAT.
C THE POSITION OF THE CURVES PARAMETERS IN THE FILE
C  IS DETERMINED BY EXAMINING THE FILE FOR CERTAIN 
C  TYPES OF LINES.
C THESE TYPES OF LINES ARE REQUIRED IN THE FILE;
C  THEY ARE FURTHER DESCRIBED IN THE MADBEND DOCUMENTATION.
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER ICOUNT, IFLAG, I
      CHARACTER*90 BUFFER
      INCLUDE 'param.h'

C --------------------------------------------------------
C OPEN THE FILE
C --------------------------------------------------------
      CALL OPENFIL(10,INFILE,'FORMATTED','READ ')

C --------------------------------------------------------
C LOCATE GLOBAL BP STEP PARAMS IF QGLOBAL.EQ.1
C --------------------------------------------------------
      IF (QGLOBAL.EQ.1) THEN
C --------------------------------------------------------
C FIND GLOBAL BP STEP PARAMS; USE UNIQUE STRING TO IDENTIFY HEADER
C   IFLAG = 0 OTHERWISE; IFLAG = 1 IF HEADER REACHED
C   ICOUNT: COUNTER TO COUNT SEVEN LINES AFTER HEADER REACHED
C --------------------------------------------------------
        IFLAG = 0
        DO WHILE (.TRUE.)
         READ(10,'(A90)',END=990) BUFFER
         IF (BUFFER(1:30).NE.'  |F| Global Inter-Base pair P') THEN
             IF (IFLAG.EQ.1) ICOUNT = ICOUNT + 1
             IF (ICOUNT.EQ.7) GOTO 5
             GOTO 9
         ELSE
             IFLAG = 1
             ICOUNT = 0
             GOTO 9
         ENDIF
       
C --------------------------------------------------------
C DATA BEGINS SEVENTH LINE AFTER HEADER REACHED
C DATA LINES IDENTIFIED AS SIXTH CHARACTER BEING ")"
C END OF DATA IDENTIFIED BY '    Average:'
C --------------------------------------------------------
5        CONTINUE
         DO WHILE (.TRUE.)
          READ(10,'(A90)',END=990) BUFFER
          IF (BUFFER(1:12).EQ.'    Average:') GOTO 10
          IF (BUFFER(6:6).EQ.')') THEN
             NUMSTEP = NUMSTEP + 1
             READ(BUFFER(45:72),*) 
     *        GTILT(NUMSTEP),GROLL(NUMSTEP),GTWIST(NUMSTEP)
          ENDIF
         END DO
  
9        CONTINUE
        END DO
      ENDIF

10    IFLAG = 0
      ICOUNT = 0
 
CDEBUG      DO I = 1, NUMSTEP
CDEBUG       WRITE(6,*) GTILT(I), GROLL(I), GTWIST(I)
CDEBUG      END DO


C --------------------------------------------------------
C  FIND LOCAL BP STEP PARAMS IF QLOCAL.EQ.1
C --------------------------------------------------------
      IF (QLOCAL.EQ.1) THEN
C --------------------------------------------------------
C FIND LOCAL BP STEP PARAMS; USE UNIQUE STRING TO IDENTIFY HEADER
C   IFLAG = 0 OTHERWISE; IFLAG = 1 IF HEADER REACHED
C   ICOUNT: COUNTER TO COUNT SEVEN LINES AFTER HEADER REACHED
C --------------------------------------------------------
        DO WHILE (.TRUE.)
         READ(10,'(A90)',END=995) BUFFER
         IF (BUFFER(1:29).NE.'  |H| Local Inter-Base pair P') THEN
             IF (IFLAG.EQ.1) ICOUNT = ICOUNT + 1
             IF (ICOUNT.EQ.7) GOTO 15
             GOTO 19
         ELSE
             IFLAG = 1
             ICOUNT = 0
             GOTO 19
         ENDIF

C --------------------------------------------------------
C DATA BEGINS SEVENTH LINE AFTER HEADER REACHED
C DATA LINES IDENTIFIED AS SIXTH CHARACTER BEING ")"
C END OF DATA IDENTIFIED BY '    Average:'
C --------------------------------------------------------
15       CONTINUE
         NUMSTEP = 0
         DO WHILE (.TRUE.)
          READ(10,'(A90)',END=995) BUFFER
          IF (BUFFER(1:12).EQ.'    Average:') GOTO 20
          IF (BUFFER(6:6).EQ.')') THEN
             NUMSTEP = NUMSTEP + 1
             READ(BUFFER(45:72),*)
     *         LTILT(NUMSTEP),LROLL(NUMSTEP),LTWIST(NUMSTEP)
          ENDIF
         END DO

19        CONTINUE
        END DO
      ENDIF

20    CONTINUE
CDEBUG      DO I = 1, NUMSTEP
CDEBUG        WRITE(6,*) GTILT(I),GROLL(I),GTWIST(I)
CDEBUG        WRITE(6,*) LTILT(I),LROLL(I),LTWIST(I)
CDEBUG      END DO
      GOTO 30

C --------------------------------------------------------
C END OF FILE ERROR MESSAGES
C --------------------------------------------------------
990   WRITE(6,*)
     *	"END OF FILE ERROR: GLOBAL CURVES PARAMETERS NOT FOUND"
      GOTO 30
995   WRITE(6,*)
     *  "END OF FILE ERROR: LOCAL CURVES PARAMETERS NOT FOUND"

30    CONTINUE
      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE PARSFLG    
C --------------------------------------------------------
C PARSE ALL COMMAND LINE ARGUMENTS
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER I, ISTART, ITERM, IEXCL, J, IORIG
      INTEGER NUMARGS, IARGC
      CHARACTER*120 TMP
      INCLUDE 'param.h'
     
      NUMARGS = 0
      QHELP = 0
      QIN = 0
      QOUT = 6
      QREF = 0
      QEXCL = 0
      QSCREEN = 0
      QLOCAL = 1
      QGLOBAL = 0
      QNOGLOB = 0
      QVERBOS = 0

C --------------------------------------------------------
C USE IARGC() TO ESTABLISH NUMBER OF COMMAND LINE ARGUMENTS
C --------------------------------------------------------
      NUMARGS = IARGC()

      DO I = 1, MAXSZ
        EXCLTLT(I) = ONE 
        EXCLROL(I) = ONE 
      END DO

C --------------------------------------------------------
C QOUT IS FLAG THAT TAKES VALUES OF 6 OR 15 - OUTPUT UNIT NUMBER
C ALL OTHER FLAGS ARE EITHER 0 OR 1 
C	INDICATING PRESENCE OF THE FOLLOWING COMMAND LINE ARGUMENTS:
C
C QHELP: 	-H        = HELP (SUPPRESSES ALL OTHER FLAGS).
C QIN: 		-I <FILE> = INPUT CURVES FILENAME.
C QOUT: 	-O <FILE> = OUTPUT FILENAME.
C QREF: 	-R <REF>  = REFERENCE POINT FOR BASE PAIR PLANE.
C QEXCL: 	-EXCL <#> <LIST> 
C			  = EXCLUDE BP STEPS, REQUIRES NUMBER AND LIST.
C QSCREEN: 	-STDOUT   = STANDARD OUTPUT ONLY; SUPPRESSES -O FLAG.
C QLOCAL: 	-NOLOCAL  = DON'T ANALYZE LOCAL BP STEP PARAMETERS; 
C		 	    LOCAL PARAMETERS ARE THE CURVES DEFAULT. 
C QGLOBAL: 	-GLOBAL   = ANALYZE GLOBAL BP STEP PARAMETERS 
C			    GLOBAL PARAMETERS ARE AN OPTION WITH CURVES INPUT 
C QNOGLOB: 	-NOGLOBAL = DON'T ANALYZE GLOBAL BP STEP PARAMETERS 
C QVERBOS: 	-VERBOSE  = VERBOSE OUTPUT
C --------------------------------------------------------

C --------------------------------------------------------
C ISTART: INDEX TO GETARG AND THE NUMBER OF COMMAND LINE ARGUMENTS
C --------------------------------------------------------
      ISTART = 0
      IF (NUMARGS.GT.0) ISTART = 1
      IF (ISTART.EQ.0) GOTO 20
      
C --------------------------------------------------------
C START OF LOOP; IORIG SAVES TEMPORARY STATE OF ISTART
C --------------------------------------------------------
15    CONTINUE
      IORIG = ISTART
      CALL GETARG(ISTART,TMP)

C --------------------------------------------------------
C QHELP: IS THERE A HELP FLAG
C --------------------------------------------------------
      IF (TMP.EQ.'-h') THEN
        QHELP = 1
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QIN: GET INPUT FILE NAME
C --------------------------------------------------------
      IF (TMP.EQ.'-i') THEN
        QIN = 1
        ISTART = ISTART + 1
        CALL GETARG(ISTART,INFILE)
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QOUT: GET OUTPUT FILE NAME; NOTE QSCREEN SUPPRESSION
C --------------------------------------------------------
      IF (TMP.EQ.'-o'.AND.QSCREEN.EQ.0) THEN
        QOUT = 15
        ISTART = ISTART + 1
        CALL GETARG(ISTART,OUTFILE)
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QSCREEN: SUPPRESSES OUTPUT FILE
C --------------------------------------------------------
      IF (TMP.EQ.'-stdout') THEN
        QSCREEN = 1
        QOUT = 6
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QVERBOS: VERBOSE OUTPUT; CAN BE USEFUL
C --------------------------------------------------------
      IF (TMP.EQ.'-verbose') THEN
        QVERBOS = 1
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QREF: GET REFERENCE PLANE PARAMETER
C --------------------------------------------------------
      IF (TMP.EQ.'-r') THEN
        QREF = 1
        ISTART = ISTART + 1
        CALL GETARG(ISTART,TMP)
        READ(TMP,*) REF
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QEXCL - GET NUMBER OF EXCLUDED TERMS 
C 
C NOTE THAT THIS CODE IS FUNCTIONALLY DUPLICATED IN
C  THE SUBROUTINE EXCLUDE.
C --------------------------------------------------------
      IF (TMP.EQ.'-excl') THEN
        QEXCL = 1
        ISTART = ISTART + 1
        CALL GETARG(ISTART,TMP)
        READ(TMP,*) NUMEXCL
        ISTART = ISTART + 1
        IF (NUMEXCL.EQ.0) GOTO 14
C --------------------------------------------------------
C READ LIST OF EXCLUDED TERMS; START PARSING LIST
C --------------------------------------------------------
        CALL GETARG(ISTART,TMP)
C --------------------------------------------------------
C COUNT NUMBER OF COMMAS AND SPACES WITH ITERM; 
C  FINAL VALUE OF ITERM SHOULD EQUAL `NUMEXCL'-1 
C --------------------------------------------------------
        ITERM=1
        J = 1
        DO I = 1, LEN(TMP)
C --------------------------------------------------------
C IF COMMA OR SPACE FOUND, INDICATES END OF TERM; 
C	PARSE PRECEDING TERM
C NON-COMMA OR SPACE CHARACTERS ARE LUMPED IN WITH NUMERIC TERMS
C    AND THUS IGNORED.
C --------------------------------------------------------
         IF (TMP(I:I).EQ.','.OR.
     *        (TMP(I:I).EQ.' '.AND.ITERM.LT.NUMEXCL)) THEN
C --------------------------------------------------------
C IEXCL: BP STEP TO EXCLUDE
C --------------------------------------------------------
           IF (TMP(J:J).NE.'r'.AND.TMP(J:J).NE.'t') THEN
             READ(TMP(J:I-1),*) IEXCL
           ELSE
             READ(TMP(J+1:I-1),*) IEXCL 
           ENDIF
           EXCLTLT(IEXCL) = ZERO
           EXCLROL(IEXCL) = ZERO
           IF (TMP(J:J).EQ.'r') EXCLTLT(IEXCL) = ONE
           IF (TMP(J:J).EQ.'t') EXCLROL(IEXCL) = ONE
           ITERM=ITERM+1
           J = I+1
         ENDIF
        END DO
C --------------------------------------------------------
C THIS CODE LEAVES LAST LIST TERM UNREAD; READ IT NOW
C --------------------------------------------------------
        IF (TMP(J:J).NE.'r'.AND.TMP(J:J).NE.'t') THEN
          READ(TMP(J:I-1),*) IEXCL
        ELSE
          READ(TMP(J+1:I-1),*) IEXCL
        ENDIF
        EXCLTLT(IEXCL) = ZERO
        EXCLROL(IEXCL) = ZERO
        IF (TMP(J:J).EQ.'r') EXCLTLT(IEXCL) = ONE
        IF (TMP(J:J).EQ.'t') EXCLROL(IEXCL) = ONE
C --------------------------------------------------------
C WARNING IF FEWER EXCLUSION LIST TERMS FOUND THAN EXPECTED
C --------------------------------------------------------
        IF (ITERM.NE.NUMEXCL) 
     *	  WRITE(6,*) "ERROR: TOO FEW EXCLUSION LIST TERMS FOUND"
        ISTART = ISTART + 1
14      CONTINUE
      ENDIF

C --------------------------------------------------------
C QNOLOCAL: DON'T USE LOCAL TERMS
C --------------------------------------------------------
      IF (TMP.EQ.'-nolocal') THEN
        QLOCAL = 0
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QGLOBAL; USE GLOBAL TERMS; NOTE QNOGLOB SUPPRESSION
C --------------------------------------------------------
      IF (TMP.EQ.'-global'.AND.QNOGLOB.EQ.0) THEN
        QGLOBAL = 1
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C QNOGLOB; DON'T USE GLOBAL TERMS; NOTE SUPPRESSION OF QGLOBAL 
C --------------------------------------------------------
      IF (TMP.EQ.'-noglobal') THEN
        QNOGLOB = 1
        QGLOBAL = 0
        ISTART = ISTART + 1
      ENDIF

C --------------------------------------------------------
C UNRECOGNIZED FLAG ERROR
C --------------------------------------------------------
      IF (ISTART.EQ.IORIG) THEN
         WRITE(6,'(A27,A120)') "ERROR: UNRECOGNIZED FLAG - ",TMP
         STOP
      ENDIF
      IF (ISTART.LE.NUMARGS) GOTO 15
20    CONTINUE

C --------------------------------------------------------
C VERBOSE STATE OUTPUT
C --------------------------------------------------------
      IF (QVERBOS.EQ.1) THEN
        WRITE(6,*) 
        WRITE(6,'(A6,I2,3X,A7,A30)') 
     *		"HELP: ",QHELP,"INPUT: ",INFILE
        WRITE(6,'(A8,I2,3X,A8,A30)') 
     *		"STDOUT: ",QSCREEN,"OUTPUT: ",OUTFILE
        WRITE(6,'(A8,F9.5,3X,A7,I2,3X,A8,I2,3X,A9,I2,3X,A10,I2)') 
     *		"REF BP: ",REF,"LOCAL: ",QLOCAL,
     *          "GLOBAL: ",QGLOBAL,"VERBOSE: ",QVERBOS,
     *		"NOGLOBAL: ",QNOGLOB
      ENDIF

      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE AFTPARS
C --------------------------------------------------------
C THIS SUBROUTINE FURTHER EXPOUNDS THE PROGRAM STATE AFTER
C	COMMAND LINE ARGUMENTS ARE PARSED.
C MADBEND WILL ASK THE USER THE VALUES OF UNDEFINED PARAMETERS.
C --------------------------------------------------------

      IMPLICIT NONE
      CHARACTER*120 EMPTY
      CHARACTER*1 YESNO
      INTEGER I, INEW
      INCLUDE 'param.h'

C --------------------------------------------------------
C INEW CONTROLS VERBOSE OUTPUT; WHEN FLAG STATES ARE CHANGED, INEW = 1
C	IF QVERBOS AND INEW ARE BOTH ONE, OUTPUT THE PROGRAM STATE
C --------------------------------------------------------
      INEW = 0
      DO I = 1,LEN(EMPTY)
       EMPTY(I:I) = ' '
      END DO

C --------------------------------------------------------
C QHELP PRINTS HELP MESSAGE AND THEN TERMINATES MADBEND
C --------------------------------------------------------
      IF (QHELP) THEN
         CALL USAGE      
         STOP
      ENDIF

C --------------------------------------------------------
C INPUT DATA FILENAME SECOND TRY
C --------------------------------------------------------
      IF (QIN.EQ.0) THEN
        WRITE(6,'(A,$)') ' WHAT IS THE INPUT DATA FILENAME: '
        READ(5,'(A120)') INFILE
        IF (INFILE.EQ.EMPTY) STOP 'ERROR READING FILENAME'
        INEW = 1
      ENDIF

C --------------------------------------------------------
C LOAD DATA; DETERMINE FORMAT OF INPUT FILE
C --------------------------------------------------------
      CALL READINP   

C --------------------------------------------------------
C REFERENCE PLANE POSITION: SECOND TRY
C --------------------------------------------------------
      IF (QREF.EQ.0) THEN
        WRITE(6,'(A,$)') ' REFERENCE BP POSITION: '
        READ(5,*) REF
        INEW = 1
      ENDIF

C --------------------------------------------------------
C IF CURVES, CHECK ON USE OF GLOBAL BP STEP PARAMETERS
C --------------------------------------------------------
      IF (QGLOBAL.EQ.0.AND.QCURVES.EQ.1.AND.QNOGLOB.EQ.0) THEN
        WRITE(6,'(A,$)') ' USE GLOBAL BP STEP TERMS [CR = NO]: '
        READ(5,'(A1)') YESNO
        IF (YESNO.EQ.'y'.OR.YESNO.EQ.'Y') QGLOBAL = 1
      ENDIF

C --------------------------------------------------------
C IF CURVES, CHECK ON STATE OF QLOCAL
C --------------------------------------------------------
      IF (QGLOBAL.EQ.1.AND.QLOCAL.EQ.1.AND.
     *		QCURVES.EQ.1.AND.QNOGLOB.EQ.0) THEN
        WRITE(6,'(A,$)') ' USE LOCAL BP STEP TERMS [CR = YES]: '
        READ(5,'(A1)') YESNO
        IF (YESNO.EQ.'n'.OR.YESNO.EQ.'N') QLOCAL = 0
      ENDIF

C --------------------------------------------------------
C OUTPUT FORMAT: SECOND TRY
C --------------------------------------------------------
      IF (QOUT.EQ.6.AND.QSCREEN.EQ.0) THEN
        WRITE(6,'(A,$)') ' OUTPUT FILENAME (CR = SCREEN): '
        READ(5,'(A120)') OUTFILE
        IF (OUTFILE.NE.EMPTY) QOUT = 15
        IF (OUTFILE.EQ.EMPTY) QSCREEN = 1
        INEW = 1
      ENDIF

C --------------------------------------------------------
C QVERBOS,INEW: WRITE PROGRAM STATE
C --------------------------------------------------------
      IF (QVERBOS.EQ.1.AND.INEW.EQ.1) THEN
        WRITE(6,*)
        WRITE(6,'(A6,I2,3X,A7,A30)') "HELP: ",QHELP,"INPUT: ",INFILE
        WRITE(6,'(A8,I2,3X,A6,A30)')
     *          "STDOUT: ",QSCREEN,"OUTPUT: ",OUTFILE
        WRITE(6,'(A8,F9.5,3X,A7,I2,3X,A8,I2,3X,A9,I2,3X,A10,I2)')
     *          "REF BP: ",REF,"LOCAL: ",QLOCAL,
     *          "GLOBAL: ",QGLOBAL,"VERBOSE: ",QVERBOS,
     *		"NOGLOBAL: ",QNOGLOB
      ENDIF

      RETURN
      END 

C****************************************************************
C****************************************************************

      SUBROUTINE USAGE      

C --------------------------------------------------------
C PRINT A USEFUL HELP MESSAGE
C --------------------------------------------------------
      
      IMPLICIT NONE
 
      WRITE(6,*) "Madbend v1.0 1/4/00"
      WRITE(6,*)
      WRITE(6,*) "This program calculates the global tilt and roll"
      WRITE(6,*) " terms associated with DNA bending."
      WRITE(6,*) 
      WRITE(6,*) "Input files and parameters may be specified as "
      WRITE(6,*) "  command line arguments, read as standard input,"
      WRITE(6,*) "  or typed as responses for the prompted quantities."
      WRITE(6,*) 
      WRITE(6,*) 
     * "Usage: -h -i <file> -o <file> -r <ref> -excl <#> <list> ",
     * " -stdout -nolocal -global -verbose"
      WRITE(6,*) "-h        help (this message); suppresses other flags"
      WRITE(6,*) "-i <file> input filename: expected to be Curves 5.2"
      WRITE(6,*) "-o <file> output filename"
      WRITE(6,*) "-r <ref>  reference bp plane position."
      WRITE(6,*) "   this uses a decimal notation." 
      WRITE(6,*) "   i.e.: 7 or 7.0 - exactly at bp 7"
      WRITE(6,*) "         7.5 - exactly between bp 7 and 8"
      WRITE(6,*) "         7.3 - closer to bp 7 (3/10) than 8 (7/10)"
      WRITE(6,*) "         7.25 - even closer to bp 7 "
      WRITE(6,*) "-excl <#> <list>  exclude bp steps from calculation"
      WRITE(6,*) "         <#> number of terms in <list> of bp steps"
      WRITE(6,*) "         <list> bp step list, t/r index tilt/roll" 
      WRITE(6,*) "         separated by comma or whitespace"
      WRITE(6,*) "     i.e. -excl 2 1,11 = ", 
     *  "exclude bp steps 1,11 (both tilt and roll)"
      WRITE(6,*) "          -excl 2 t1,r11 = exclude tilt 1, roll 11"
      WRITE(6,*) "          -excl 2 1,r11 = ",
     *  "exclude tilt and roll 1, roll 11"
      WRITE(6,*) "          -excl 0 = use all bp step parameters"
      WRITE(6,*) "-stdout   standard output; suppress -o flag"
      WRITE(6,*) "-global   use global bp step parameters"
      WRITE(6,*) "-nolocal  don't use local bp step parameters"
      WRITE(6,*) "-verbose  verbose output"
      WRITE(6,*)


      RETURN
      END

C****************************************************************
C****************************************************************

      SUBROUTINE OPENFIL(INUM,FILENAM,FILETYP,WRITEST)

C --------------------------------------------------------
C THIS IS A GENERALIZED FILE OPENING SUBROUTINE
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER INUM,IOS
      CHARACTER*120 FILENAM
      CHARACTER*11 FILETYP
      CHARACTER*5 WRITEST

C --------------------------------------------------------
C WRITEST CONTROLS FILE OPENING STATE: READ OR WRITE 
C --------------------------------------------------------
      IF (WRITEST.EQ.'READ ') THEN
       OPEN(UNIT=INUM,FILE=FILENAM,FORM=FILETYP,IOSTAT=IOS,READONLY)
      ELSE IF (WRITEST.EQ.'WRITE') THEN
       OPEN(UNIT=INUM,FILE=FILENAM,FORM=FILETYP,IOSTAT=IOS)
      ENDIF

C --------------------------------------------------------
C IOS: I/O ERROR STATE
C --------------------------------------------------------
      IF (IOS.EQ.0) THEN
CDEBUG       WRITE(6,'(A,A120)') 'SUCCESSFULLY OPENED ',FILENAM
       RETURN
      ELSE
       WRITE(6,'(A15,A120)') 'ERROR OPENING: ',FILENAM
       STOP
      ENDIF

      END

C****************************************************************
C****************************************************************

      SUBROUTINE OUTGTGR    

C --------------------------------------------------------
C THIS IS THE OUTPUT SUBROUTINE FOR THE CALCULATED
C  VALUES OF GLOBAL TILT AND GLOBAL ROLL
C --------------------------------------------------------

      IMPLICIT NONE
      INTEGER I
      DOUBLE PRECISION SUM, GTSUM, GRSUM
      INCLUDE 'param.h'

C --------------------------------------------------------
C OPEN OUTPUT FILE IF OUTPUT UNIT IS 15
C IF THE OUTPUT UNIT IS 6, USE STANDARD OUTPUT 
C --------------------------------------------------------
      IF (QOUT.EQ.15) THEN
        CALL OPENFIL(QOUT,OUTFILE,'FORMATTED  ','WRITE')
      ENDIF 

C --------------------------------------------------------
C WRITE OUT THE HEADER MESSAGE
C --------------------------------------------------------
      WRITE(QOUT,*) 
      WRITE(QOUT,'(A7)') 'OUTPUT:' 

C --------------------------------------------------------
C IF VERBOSE OUTPUT IS REQUESTED
C  WRITE A DETAILED OUTPUT, INCLUDING THE INDIVIDUAL TERMS 
C   OF GLOBAL TILT AND GLOBAL ROLL
C --------------------------------------------------------
      IF (QVERBOS.EQ.1) THEN
C --------------------------------------------------------
C WRITE INDIVIDUAL TERMS OF GLOBAL TILT/ROLL SUM
C --------------------------------------------------------
       WRITE(QOUT,*)
C --------------------------------------------------------
C LOCAL TILT/ROLL  TERMS
C --------------------------------------------------------
       IF (QLOCAL.EQ.1) THEN
         WRITE(QOUT,'(A40)') 'INDIVIDUAL TERMS USING LOCAL PARAMETERS:'
         WRITE(QOUT,'(5X,A11,6X,A11)') 'GLOBAL TILT', 'GLOBAL ROLL'
         DO I = 1, NUMTERM
           WRITE(QOUT,'(I5,3X,F11.6,5X,F11.6)') I,GTL(I),GRL(I)
         END DO
       ENDIF

C --------------------------------------------------------
C GLOBAL TILT/ROLL TERMS
C --------------------------------------------------------
       IF (QGLOBAL.EQ.1) THEN
         WRITE(QOUT,'(A41)') 'INDIVIDUAL TERMS USING GLOBAL PARAMETERS:'
         DO I = 1, NUMTERM
          WRITE(QOUT,'(I5,3X,F11.6,5X,F11.6)') I,GTG(I),GRG(I)
         END DO
       ENDIF
      ENDIF
C --------------------------------------------------------
C END VERBOSE SECTION
C --------------------------------------------------------


C --------------------------------------------------------
C THIS SECTION OUTPUTS THE CUMULATIVE GLOBAL TILT AND GLOBAL ROLL
C  IT'S UNAFFECTED BY QVERBOS.
C --------------------------------------------------------
C --------------------------------------------------------
C GT/GR TOTALS AND TOTAL BEND
C    LOCAL
C --------------------------------------------------------
      IF (QLOCAL.EQ.1) THEN
        WRITE(QOUT,'(A40)') 'CUMULATIVE TERMS USING LOCAL PARAMETERS:'
        SUM = ZERO
        GTSUM = ZERO
        GRSUM = ZERO
        DO I = 1, NUMTERM
          GTSUM = GTSUM + GTL(I)
          GRSUM = GRSUM + GRL(I)
        END DO
        SUM = SQRT(GTSUM**2 + GRSUM**2)
        WRITE(QOUT,10) SUM , GTSUM, GRSUM
10      FORMAT('TOTAL BEND: ',F11.6,5X,'GLOBAL TILT: ',F11.6,5X,
     *		'GLOBAL ROLL: ',F11.6)
      ENDIF
     

C --------------------------------------------------------
C    GLOBAL
C --------------------------------------------------------
      IF (QGLOBAL.EQ.1) THEN
        WRITE(QOUT,'(A41)') 'CUMULATIVE TERMS USING GLOBAL PARAMETERS:'
        SUM = ZERO
        GTSUM = ZERO
        GRSUM = ZERO
        DO I = 1, NUMTERM
          GTSUM = GTSUM + GTG(I)
          GRSUM = GRSUM + GRG(I)
        END DO
        SUM = SQRT(GTSUM**2 + GRSUM**2)
        WRITE(QOUT,10) SUM , GTSUM, GRSUM
      ENDIF

C --------------------------------------------------------
C CONTRIBUTION OF INDIVIDUAL TERMS TO GT/GR TOTALS
C --------------------------------------------------------
C      WRITE(QOUT,'(//)')
C      WRITE(QOUT,'(A46)') 
C     *		'EFFECT OF INDIVIDUAL TERMS ON GLOBAL TILT/ROLL'
C      WRITE(QOUT,'(A12)') 'LOCAL TERMS:'
C      GTSUM = ZERO
C      GRSUM = ZERO
C      DO I = 1, NUMTERM
C        GTSUM = GTSUM + GTL(I)
C        GRSUM = GRSUM + GRL(I)
C      END DO
C      DO I = 1, NUMTERM
C        WRITE(QOUT,15) GTSUM-GTL(I), GRSUM, GTSUM, GRSUM - GRL(I),
C     *		GTSUM-GTL(I), GRSUM-GRL(I)
C      END DO
C15    FORMAT(F9.5,5X,F9.5,5X,F9.5,5X,F9.5,5X,F9.5,5X,F9.5)

      RETURN
      END
